/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __maya_MayaNode_h__
#define __maya_MayaNode_h__

namespace fe
{

/**************************************************************************//**
    @brief Maya node

	@ingroup maya
*//***************************************************************************/
class MayaNode:
	public MPxNode,
	public MetaPlugin
{
	private:
	class Parameter
	{
		public:
			MObject						m_object;
			MObject						m_objectPaint;
			MObject						m_objectPaintList;
	};
	class Registration
	{
		public:
			U32							m_uniqueId;
			String						m_implementation;
			std::map<String,Parameter>	m_parameterMap;
			std::vector<String>			m_inputChannelArray;
			std::vector<String>			m_outputChannelArray;
			BWORD						m_autoOutput;
	};
	class Brush
	{
		public:
			MayaNode*					m_pMayaNode;
	};

	public:
								MayaNode(void);
virtual							~MayaNode(void);

virtual	void					dirty(void);

virtual	void					postConstructor(void);

static	void*					create(void);

static	void					preRegister(U32 a_uniqueId,String a_name,
										String a_implementation);
static	MStatus					initialize(void);
static	void					postRegister(void);

virtual	MStatus					compute(const MPlug& a_rPlug,
										MDataBlock& a_rData);
virtual MStatus					shouldSave(const MPlug& a_rPlug,
										bool& a_rResult);

								///	@internal
		void					brush(WindowEvent& a_rEvent,
										const SpatialVector& a_rRayOrigin,
										const SpatialVector& a_rRayDirection);

static	std::map<String,Brush>			ms_brushMap;

	private:
		void							updateComponents(BWORD a_constructing);
		void							precompute(Real a_time);
static	void							populateOperator(String a_name);
		void							updateCatalog(MDataBlock& a_rData,
											Real a_time,BWORD a_doInput);
		void							relayErrors(void);
		void							restoreState(void);

static	I32								ms_registerLock;	//* TODO threadsafe
static	String							ms_registerName;

static	MObject							ms_compoundBehavior;

static	std::map<String,Registration>	ms_mayaRegistry;

		MayaContext						m_mayaContext;

		String							m_typeName;
		String							m_nodeName;
		String							m_implementation;

		U32								m_paintSize;
		BWORD							m_bound;
};

} // namespace

#endif /* __maya_MayaNode_h__ */
